@@ -4,131 +4,77 @@
  * user-provided command-line settings.                                    *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
- * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we sell    *
- * alternative licenses (contact sales@insecure.com).  Dozens of software  *
- * vendors already license Nmap technology such as host discovery, port    *
- * scanning, OS detection, version detection, and the Nmap Scripting       *
- * Engine.                                                                 *
- *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
- *                                                                         *
- * o Integrates source code from Covered Software.                         *
- *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
- *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
- *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, Insecure.Com LLC grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the special and conditions of the license text as well.       *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
- *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2024 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: NmapOps.h 31563 2013-07-28 22:08:48Z fyodor $ */
+/* $Id: NmapOps.h 38790 2024-02-28 18:46:45Z dmiller $ */
 
-#include "nmap.h"
-#include "global_structures.h"
-#include "output.h"
+#ifndef NMAP_OPS_H
+#define NMAP_OPS_H
+
+#include "nmap.h" /* MAX_DECOYS */
+#include "scan_lists.h"
+#include "output.h" /* LOG_NUM_FILES */
+#include <nbase.h>
 #include <nsock.h>
 #include <string>
+#include <map>
+#include <vector>
+
+struct FingerPrintDB;
+struct FingerMatch;
 
 class NmapOps {
  public:
@@ -167,14 +113,15 @@ class NmapOps {
      is not necessarily used. */
   bool RawScan();
   void ValidateOptions(); /* Checks that the options given are
-                             reasonable and consistant.  If they aren't, the
+                             reasonable and consistent.  If they aren't, the
                              function may bail out of Nmap or make small
-		             adjustments (quietly or with a warning to the
-		             user). */
+                             adjustments (quietly or with a warning to the
+                             user). */
   int isr00t;
   /* Whether we have pcap functions (can be false on Windows). */
   bool have_pcap;
-  int debugging;
+  u8 debugging;
+  bool resuming;
 
 #define PACKET_SEND_NOPREF 1
 #define PACKET_SEND_ETH_WEAK 2
@@ -206,21 +153,21 @@ class NmapOps {
   void setVersionTrace(bool vt) { vTrace = vt;  }
   bool openOnly() { return open_only; }
   void setOpenOnly(bool oo) { open_only = oo; }
-  int verbose;
+  u8 verbose;
   /* The requested minimum packet sending rate, or 0.0 if unset. */
   float min_packet_send_rate;
   /* The requested maximum packet sending rate, or 0.0 if unset. */
   float max_packet_send_rate;
   /* The requested auto stats printing interval, or 0.0 if unset. */
   float stats_interval;
-  int randomize_hosts;
-  int randomize_ports;
-  int spoofsource; /* -S used */
-  int fastscan;
+  bool randomize_hosts;
+  bool randomize_ports;
+  bool spoofsource; /* -S used */
+  bool fastscan;
   char device[64];
   int ping_group_sz;
-  int nogcc; /* Turn off group congestion control with --nogcc */
-  int generate_random_ips; /* -iR option */
+  bool nogcc; /* Turn off group congestion control with --nogcc */
+  bool generate_random_ips; /* -iR option */
   FingerPrintDB *reference_FPs; /* Used in the new OS scan system. */
   std::vector<FingerMatch> os_labels_ipv6;
   u16 magic_port; /* The source port set by -g or --source-port. */
@@ -241,7 +188,7 @@ class NmapOps {
 
   /* These functions retrieve and set the Round Trip Time timeouts, in
    milliseconds.  The set versions do extra processing to insure sane
-   values and to adjust each other to insure consistance (e.g. that
+   values and to adjust each other to insure consistence (e.g. that
    max is always at least as high as min) */
   int maxRttTimeout() { return max_rtt_timeout; }
   int minRttTimeout() { return min_rtt_timeout; }
@@ -279,7 +226,7 @@ class NmapOps {
   /* Gets the spoofed MAC address, but returns NULL if it hasn't been set */
   const u8 *spoofMACAddress() { return spoof_mac_set? spoof_mac : NULL; }
 
-  int max_ips_to_scan; // Used for Random input (-iR) to specify how 
+  unsigned int max_ips_to_scan; // Used for Random input (-iR) to specify how
                        // many IPs to try before stopping. 0 means unlimited.
   int extra_payload_length; /* These two are for --data-length op */
   char *extra_payload;
@@ -289,34 +236,38 @@ class NmapOps {
   bool open_only;
 
   int scanflags; /* if not -1, this value should dictate the TCP flags
-		    for the core portscaning routine (eg to change a
-		    FIN scan into a PSH scan.  Sort of a hack, but can
-		    be very useful sometimes. */
+                    for the core portscanning routine (eg to change a
+                    FIN scan into a PSH scan.  Sort of a hack, but can
+                    be very useful sometimes. */
 
-  int defeat_rst_ratelimit; /* Solaris 9 rate-limits RSTs so scanning is very
+  bool defeat_rst_ratelimit; /* Solaris 9 rate-limits RSTs so scanning is very
             slow against it. If we don't distinguish between closed and filtered ports,
             we can get the list of open ports very fast */
 
-  struct in_addr resume_ip; /* The last IP in the log file if user 
-			       requested --restore .  Otherwise 
-			       restore_ip.s_addr == 0.  Also 
-			       target_struct_get will eventually set it 
-			       to 0. */
+  bool defeat_icmp_ratelimit; /* If a host rate-limits ICMP responses, then scanning
+            is very slow against it. This option prevents Nmap to adjust timing
+            when it changes the port's state because of ICMP response, as the latter
+            might be rate-limited. Doing so we can get scan results faster. */
+
+  struct sockaddr_storage resume_ip; /* The last IP in the log file if user
+                               requested --restore .  Otherwise
+                               resume_ip.ss_family == AF_UNSPEC.  Also
+                               Target::next_target will eventually set it
+                               to AF_UNSPEC. */
 
   // Version Detection Options
-  int override_excludeports;
+  bool override_excludeports;
   int version_intensity;
 
-  struct in_addr decoys[MAX_DECOYS];
-  int osscan_limit; /* Skip OS Scan if no open or no closed TCP ports */
-  int osscan_guess;   /* Be more aggressive in guessing OS type */
+  struct sockaddr_storage decoys[MAX_DECOYS];
+  bool osscan_limit; /* Skip OS Scan if no open or no closed TCP ports */
+  bool osscan_guess;   /* Be more aggressive in guessing OS type */
   int numdecoys;
   int decoyturn;
-  int osscan;
-  int servicescan;
+  bool osscan;
+  bool servicescan;
   int pingtype;
   int listscan;
-  int allowall;
   int fragscan; /* 0 or MTU (without IPv4 header size) */
   int ackscan;
   int bouncescan;
@@ -333,13 +284,13 @@ class NmapOps {
   int sctpcookieechoscan;
   int windowscan;
   int xmasscan;
-  int noresolve;
-  int noportscan;
-  int append_output; /* Append to any output files rather than overwrite */
+  bool noresolve;
+  bool noportscan;
+  bool append_output; /* Append to any output files rather than overwrite */
   FILE *logfd[LOG_NUM_FILES];
   FILE *nmap_stdout; /* Nmap standard output */
   int ttl; // Time to live
-  int badsum;
+  bool badsum;
   char *datadir;
   /* A map from abstract data file names like "nmap-services" and "nmap-os-db"
      to paths which have been requested by the user. nmap_fetchfile will return
@@ -350,7 +301,9 @@ class NmapOps {
      Only files that were actually read should be in this map. */
   std::map<std::string, std::string> loaded_data_files;
   bool mass_dns;
-  int resolve_all;
+  bool always_resolve;
+  bool resolve_all;
+  bool unique;
   char *dns_servers;
 
   /* Do IPv4 ARP or IPv6 ND scan of directly connected Ethernet hosts, even if
@@ -372,17 +325,20 @@ class NmapOps {
   char *exclude_spec;
   FILE *inputfd;
   char *portlist; /* Ports list specified by user */
+  char *exclude_portlist; /* exclude-ports list specified by user */
 
   nsock_proxychain proxy_chain;
+  bool discovery_ignore_rst; /* host discovery should not consider TCP RST packet responses as a live asset */
 
 #ifndef NOLUA
-  int script;
+  bool script;
   char *scriptargs;
   char *scriptargsfile;
-  int scriptversion;
-  int scripttrace;
-  int scriptupdatedb;
+  bool scriptversion;
+  bool scripttrace;
+  bool scriptupdatedb;
   bool scripthelp;
+  double scripttimeout;
   void chooseScripts(char* argument);
   std::vector<std::string> chosenScripts;
 #endif
@@ -394,11 +350,12 @@ class NmapOps {
   int ipopt_lasthop;	// offset in ipoptions where is space for targets ip for source/strict routing
 
   // Statistics Options set in nmap.cc
-  int numhosts_scanned;
-  int numhosts_up;
+  unsigned int numhosts_scanned;
+  unsigned int numhosts_up;
   int numhosts_scanning;
   stype current_scantype;
   bool noninteractive;
+  char *locale;
 
   bool release_memory;	/* suggest to release memory before quitting. used to find memory leaks. */
  private:
@@ -413,7 +370,7 @@ class NmapOps {
   unsigned int min_host_group_sz;
   unsigned int max_host_group_sz;
   void Initialize();
-  int addressfamily; /*  Address family:  AF_INET or AF_INET6 */  
+  int addressfamily; /*  Address family:  AF_INET or AF_INET6 */
   struct sockaddr_storage sourcesock;
   size_t sourcesocklen;
   struct timeval start_time;
@@ -424,4 +381,5 @@ class NmapOps {
   u8 spoof_mac[6];
   bool spoof_mac_set;
 };
-  
+
+#endif
